/*************************************************************
* archdep.h Things that change depending on the archetecture
* Copyright (C) 2006 (April, 4th) OpenWii (admin@openwii.org)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* WiiP! (AT90S2313) AVR Flavor - eris
**************************************************************/

#ifndef ARCHDEP_H
#define ARCHDEP_H

/*Other files to include for this micro*/
#include <avr/io.h>
#include <avr/sfr_defs.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <inttypes.h>
//#include <avr/boot.h>

/*Pin definitions*/
#define PIN_CLK (_BV(PB0))	//PORT MASK FOR CLK PIN
#define PIN_OUT (_BV(PB1))	//PORT MASK FOR OUTPUT PIN
#define PIN_IN	(_BV(PB2))	//PORT MASK FOR INPUT PIN
#define PIN_SW	(_BV(PD6))	//PORT MASK FOR SWITCH (used to reset eeprom to factory defaults)

/*macros to change the output pin states*/
#define OUTHIZ DDRB &= ~PIN_OUT // Output HiZi
#define OUTLOW DDRB |= PIN_OUT	// Output Low
#define CLKHIZ DDRB &= ~PIN_CLK // clock HiZ
#define CLKLOW DDRB |= PIN_CLK //Drive clock low

/*Macro to read input pin as 0 or 1*/
#define GETIN ((PINB & PIN_IN) >> 2) // read input 

/*Macros for LED*/
#define LEDPIN (_BV(PB4))
#define LEDON	PORTB&=~LEDPIN
#define LEDOFF	PORTB|=LEDPIN
#define LEDTOG  PORTB^=LEDPIN

/*Macros to define switch input*/
#define SW0IN ((PIND >> 6) & 0x1)

/*Define the clock feq for avr*/
#define F_CPU 4000000
#define USART_BAUD 9600 /*setup the uart just in case*/
#define USART_UBBR_VALUE ((F_CPU/(USART_BAUD<<4))-1)

/*Stuff used by the timer*/
//#define USTIMERVAL 100000	// 1/.000001 
#define USTIMERVAL 1000000
#define MSTIMERVAL 1000		// 1/.001
#define USTIMERDIV ((F_CPU/USTIMERVAL)-1)
#define MSTIMERDIV ((F_CPU/MSTIMERVAL)-1)

/*These are just macros to define what functions to use for eeprom and program memory access*/
#define nvReadByte eeprom_read_byte
#define nvWriteByte eeprom_write_byte
#define pgmReadWord pgm_read_word

#define PAGESIZE 16
#define MAXPAGE 64
#define MIRRORADDR 2048
#define MAXFIRMLEN 3580
#define APPLADDR 0x0000
#define BOOTLADDR 0x0000


#  if defined(GET_LOCK_BITS)    /* avr-libc >= 1.2.5 */
#    define _GET_LOCK_BITS() boot_lock_fuse_bits_get(GET_LOCK_BITS)
#    define _GET_LOW_FUSES() boot_lock_fuse_bits_get(GET_LOW_FUSE_BITS)
#    define _GET_HIGH_FUSES() boot_lock_fuse_bits_get(GET_HIGH_FUSE_BITS)
#    define _GET_EXTENDED_FUSES() boot_lock_fuse_bits_get(GET_EXTENDED_FUSE_BITS)
#  endif /* defined(GET_LOCK_BITS) */
#  define _SET_LOCK_BITS(data) boot_lock_bits_set(~data)
#  define _ENABLE_RWW_SECTION() boot_rww_enable()

#  define _WAIT_FOR_SPM() boot_spm_busy_wait()

#  ifndef LARGE_MEMORY
#    define _LOAD_PROGRAM_MEMORY(addr) pgm_read_byte_near(addr)
#  else /* LARGE_MEMORY */
#    define _LOAD_PROGRAM_MEMORY(addr) pgm_read_byte_far(addr)
#  endif /* LARGE_MEMORY */
#  define _addToPageBuffer(addr,data) boot_page_fill(addr, data)
#  define _PAGE_ERASE(addr) boot_page_erase(addr)
#  define _PAGE_WRITE(addr) boot_page_write(addr)



#endif
