/*************************************************************
* bootloader.c - openwii umod2, elegantly simple version
* Copyright (C) 2007 (April, 24th) OpenWii (admin@openwii.org)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
**************************************************************/

#include "common.h"
#define CHECKSUM
//#define DBGADDR 0x1700
__attribute__ ((section (".bootloader_func"))) uint8_t BlSerialRequest(uint8_t command, uint32_t offset, uint8_t size, uint16_t data, uint16_t *ret);
__attribute__ ((section (".bootloader_func"))) void wrtPage(uint16_t addr);
__attribute__ ((section (".bootloader_func"))) void waitRWW(void);
__attribute__ ((section (".bootloader_func"))) void Blwaitus(uint16_t us);
__attribute__ ((section (".bootloader_func"))) uint16_t readInt(uint16_t addr);


__attribute__ ((section (".bootloader"))) void loader()
{
	uint16_t ret_word;
	uint16_t firm_len;// = 1938;
	uint16_t nb_recv;
	uint16_t crc16_calc;
	uint16_t crc16_lu;
	uint16_t addr,flash_addr;
        uint8_t sreg;
        // Disable interrupts.
        sreg = SREG;
        cli();
        LEDOFF;
//        LEDON;
	// retrieve the length of the firmware (2 bytes len + firmware + 2 crc16 bytes)
	// len is only the length of the firmware itself	
	BlSerialRequest(CMD_READ, FLASH_UPDT_ADDR, 2, 0x0, &ret_word);
	//firm_len = ((((ret_word) & 0xFF) << 8) & 0xFF00) + ((ret_word >> 8) & 0xFF);
	firm_len = readInt(ret_word);
	nb_recv = 0;
	crc16_calc=0;
	addr = MIRRORADDR;
	while(nb_recv < firm_len) {
		/*Read the firmware word by word*/
		BlSerialRequest(CMD_READ, FLASH_UPDT_ADDR + nb_recv + 2, 2, 0x0, &ret_word);
		nb_recv += 2;
#ifdef CHECKSUM
		// Calculate the crc16
		crc16_calc = crc16_calc + ((ret_word >> 8) & 0xFF) + (ret_word & 0xFF);
#endif		
		// call the self programming routines
		_addToPageBuffer(addr,ret_word);
		_WAIT_FOR_SPM();
		if ((nb_recv % PAGESIZE) == 0) {
			// call the write page routine
			LEDTOG;
			wrtPage(addr);
		}
		addr += 2;
	}
	if (!((nb_recv % PAGESIZE) == 0)) {
		// call the write page routine for the last page
//		LEDTOG;
		wrtPage(addr);
	}
#ifdef CHECKSUM
	// Read the transmitted CRC16	
	BlSerialRequest(CMD_READ, FLASH_UPDT_ADDR + nb_recv + 2, 2, 0x0, &ret_word);
	crc16_lu = readInt(ret_word);
	// Control the CRC16 transmitted versus the one calculated
	// commented for testing if (!(ret_word = crc16)) return;
	//	_addToPageBuffer(DBGADDR,crc16_calc);
	//	_WAIT_FOR_SPM();
	//	_addToPageBuffer(DBGADDR+2,crc16_lu);
	//	_WAIT_FOR_SPM();
	//	wrtPage(DBGADDR);
	
	if (!(crc16_calc == crc16_lu)) {
		// exit without flashing
		// perhaps return an error to the GC appl
		//LEDON;
		goto endofjob;
	}
	
#endif
	// if the crc is OK, proceed to the real programming
	// read the mirrored flash and write it in the real flash
	//LEDOFF;
	//waitus(2000000);
	nb_recv = 0;
	flash_addr = APPLADDR;
	addr = MIRRORADDR;
//	LEDOFF;
	while(nb_recv < firm_len) {
		/*Read the firmware word by word in the mirror adress space*/
		waitRWW();
		ret_word = ((_LOAD_PROGRAM_MEMORY(addr+1)) << 8) | ((_LOAD_PROGRAM_MEMORY(addr+0)) & 0xFF);
		nb_recv += 2;
		// call the self programming routines
		_addToPageBuffer(flash_addr,ret_word);
		_WAIT_FOR_SPM();
		if ((nb_recv % PAGESIZE) == 0) {
			// call the write page routine
			LEDTOG;
			wrtPage(flash_addr);
		}
		addr += 2;
		flash_addr +=2;
	}
	if (!((nb_recv % PAGESIZE) == 0)) {
		// call the write page routine for the last page
//		LEDTOG;
		wrtPage(flash_addr);
	}
	// TO DO : Adress how to program the bootloader itself, but is it necessary ??
        //LEDOFF;
#ifdef CHECKSUM
endofjob:	
#endif	
	// return to application
        LEDOFF;
	_WAIT_FOR_SPM();
	waitRWW();
	BlSerialRequest(CMD_WRITE, CMD_UPDT_ADDR, 2, 0x0, NULL);
        // restore interrupt if necessary
        SREG = sreg;
	//jump_to_application();
	asm volatile("push r0\n\t"
		     "clr r0\n\t"
		     "mov r31,r0\n\t"
		     "mov r30,r0\n\t"
		     "pop r0\n\t"
		     "ijmp\n\t"
		     ::);
}

__attribute__ ((section (".bootloader_func"))) void wrtPage(uint16_t addr)
{
		_PAGE_ERASE(addr);
		_WAIT_FOR_SPM();
		_PAGE_WRITE(addr);
		_WAIT_FOR_SPM();
}

__attribute__ ((section (".bootloader_func"))) void waitRWW(void)
{
		while (_IS_RWW_BUSY) {
			_ENABLE_RWW_SECTION();
		}
}

__attribute__ ((section (".bootloader_func"))) uint16_t readInt(uint16_t lu)
{
	uint16_t result;
	result = ((((lu) & 0xFF) << 8) & 0xFF00) + ((lu >> 8) & 0xFF);
	return result;
}

/* wait approx 1 us */
__attribute__ ((section (".bootloader_func"))) void Blwaitus(uint16_t us)
{
	int16_t t0=TCNT1;
	us*=(uint8_t)USTIMERDIV; //us now = how long 1 ms takes to pass (this requires mathlibs and more space...)
	while((TCNT1-t0) < us); //only works correctly if timer is not in compare mode -krall
}

/*xmit out 1 byte via serial interface*/
__attribute__ ((section (".bootloader_func"))) void BlTxByte(uint8_t byte)
{
	int y;
	for(y=0;y<8;y++)
	{
		CLKLOW;	// Drive clock low
		Blwaitus(SERCLKDLY);
		if(byte & 0x1 )
		{
			OUTHIZ;	// Make output pin highZ 
		}else{
			OUTLOW;	// Drive output pin low
		}
		byte>>=1;
		CLKHIZ;	// HighZ clock pin
		Blwaitus(SERCLKDLY);
	}
	Blwaitus(100);
	CLKLOW;		// Drive clock low
	OUTLOW;		// Drive output low
}

/*rcv 1 byte via serial*/
__attribute__ ((section (".bootloader_func"))) uint8_t BlRxByte()
{
	uint8_t byte=0;
	uint8_t y;
	for(y=0;y<8;y++)
	{
		CLKLOW;
		Blwaitus(SERCLKDLY);
		byte |= GETIN << y;
		CLKHIZ;
		Blwaitus(SERCLKDLY);
	}
	Blwaitus(100);
	CLKLOW;	
	return byte;
}

/*Activate a serial command*/
__attribute__ ((section (".bootloader_func"))) uint8_t BlSerialRequest(uint8_t command, uint32_t offset, uint8_t size, uint16_t data, uint16_t *ret)
{
	uint8_t Buffer[10];		// Command Buffer
	uint8_t x;
	uint8_t rb0,rb1;

	Buffer[0] = command;
	Buffer[1] = 0;
	Buffer[2] = (char) ((offset >> 8) & 0xFF);
	Buffer[3] = (char) (offset & 0xFF);
	Buffer[4] = (char) (data & 0xFF);
	Buffer[5] = (char) ((offset >> 16) & 0xFF);
	Buffer[6] = (char) ((data >> 8) & 0xFF);
	Buffer[7] = 0;
	Buffer[8] = size;
	Buffer[9] = 0;

	for(x=0;x<SERIAL_CMD_SIZE;x++)
	{
		BlTxByte(Buffer[x]);
	}
	Blwaitus(1000);
	// For byte reads, zero the last result byte so the mask below won't contain old data
	rb1=0;
	// Get and check manditory part of the reply.
	if(BlRxByte() | BlRxByte()) 
		return -1;
	rb0=BlRxByte();
	if(size==2)
		rb1=BlRxByte();
	if(Buffer[0] == CMD_WRITE) //if write
	{ 
		if(Buffer[4] != rb0) //check that the first byte was written
			return -1;
		if( (size == 2) && (Buffer[6] != rb1))	// check that the next byte was written if writing 2 bytes (what about >2 bytes??)
			return -1;
	}	
	if(Buffer[0] == CMD_READ) // if reading copy the result buffer to the return buffer
	{
		*ret = ((uint16_t)rb0) | ((uint16_t)(rb1 << 8) & 0xFF00);
	}
	return 0;
}
