/*************************************************************
* common.h - this file selects which micro specific stuff to include and prototypes micro specific functions.
* Copyright (C) 2006 (April, 4th) OpenWii (admin@openwii.org)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
**************************************************************/

#ifndef COMMON_H
#define COMMON_H
#include <stdlib.h>

/* Include archdep for whatever chip/conf u have */
#ifdef msp430
#include "../msp430/archdep.h"
#endif

#ifdef chiip
#include "../chiip/archdep.h"
#endif

#ifdef avr
#include "../avr/archdep.h"
#endif

#include "serial.h"

/*These functions should be defined in archdep.c or archdep.S if ur an asm freak :)*/
/*note this function is only needed if u havent allready #defined it to something else. This allows for using an inline macro (as is supplied in the avr's headers with eeprom_read_byte, pgm_read_word, etc).*/
#ifndef nvReadByte
inline uint8_t nvReadByte(uint16_t); //read a byte from non-vol memory (eeprom, flash, whatever)
#endif
#ifndef nvWriteByte
inline void nvWriteByte(uint16_t,uint8_t); //write a byte to non-vol memory
#endif
#ifndef pgmReadWord
inline uint16_t pgmReadWord(PGM_P); //read a word (16 bit) from program memory
#endif
inline void machInit(void); //Setup machiene dependand stuff not covered elsewhere
inline void waitus(uint16_t us); //machiene dependant function to wait approx 1ms
inline void msTimerInt(); // This function is called by the timer interrupt (for leds mostly)
//extern void loader();

/*These are used by both the mn10200 and modchip and come from the jsym file.*/
/*dunno if ill use them or not...*/
extern unsigned char *_memmap __attribute__ ((section ("*ABS*")));


/*We dont even use these but hey, they are here..*/
#define GCFORMAT 	0x0040 gc format
#define WIIFORMAT 	0x0080 wii format
#define NOTSINGLE	0x0100 0-single
#define PTPDISC		0x0200 ptp disc
#define OTPDISC		0x0400 otp disc
#define REALDISC	0x0800 real disc
#define EMUDISC		0x1000 emu disc
#define DISC120		0x4000 0=80mmdisc,1=120mm

/*these should be defined in a single place and included to whatever*/
#define JAP_R 0x00 //is JAP
#define USA_R 0x01 //is USA
#define PAL_R 0x02 //is PAL
#define OFF_R 0x04 //is OFF (default region)

#define REGXFERADDR	0x8100	//what address is used to transfer region data to and from dvd drive from microcontroller
#define EEREGADDR 	0x00	//The address in eeprom to used for storing region and other flags
#define EEERASEDSTATE	0xff	//erased value of eeprom data

#ifdef dms
/*these should be defined by the symbol file..
  but note, if the microcontroller is 16 bit addressing then it may truncate 32 bit addresses to 16 bits. 
  iotw, 0x40bd7a on a AVR will end up as bd7a.. so only 16 bit addresses will work with avr*/
#define UNLOCKADDR 	0x82b6	//p_unlockAddr=0x82b6;
#define STACKRET	0x8f84	//p_stackRet = 0x8f84;
#define ORGRET		0xd4c7	//org_ret=0xd4c7;

/*These symbols are filled in by the linker and give us the starting location and size of the raw mn10200 binary file*/
//#define BINARYSTART _binary_dms_bin_start
//#define BINARYSIZE _binary_dms_bin_size
#endif

#ifdef d2b
#define UNLOCKADDR 	0x82be	//p_unlockAddr=0x82b6;
#define STACKRET	0x8f84	//p_stackRet = 0x8f84;
#define ORGRET		0xd54a	//org_ret=0xd4c7;

//#define BINARYSTART _binary_dms_raw_start
//#define BINARYSIZE _binary_dms_raw_size
#endif

// For bootloader
#define CMD_UPDT_ADDR 0x8120    // what adress is used to send th eupdate flash command
#define FLASH_UPDT_ADDR 0x40C000  // what adress is used to transfer firmware to program
#define CMD_LOAD 0x09
#define DEBUG_ADDR 0x8150

extern void *BINARYSTART;
extern const void BINARYSIZE;

#endif /*COMMON_H*/
