/******************************************************************
* serial.c - OpenWii DVD Drive CN302 serial port access functions
* Copyright (C) 2006 (April, 4th) OpenWii (admin@openwii.org)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* version 2.0 the simple and elegant version
*******************************************************************/


#include "common.h"

uint8_t Buffer[10];				// Command Buffer
uint8_t y;

/*xmit out 1 byte via serial interface*/
void TxByte(uint8_t byte)
{
	for(y=0;y<8;y++)
	{
		CLKLOW;	// Drive clock low
		waitus(SERCLKDLY);
		if(byte & 0x1 )
		{
			OUTHIZ;	// Make output pin highZ 
		}else{
			OUTLOW;	// Drive output pin low
		}
		byte>>=1;
		CLKHIZ;	// HighZ clock pin
		waitus(SERCLKDLY);
	}
	waitus(100);
	CLKLOW;		// Drive clock low
	OUTLOW;		// Drive output low
}

/*rcv 1 byte via serial*/
uint8_t RxByte()
{
	uint8_t byte=0;
//	uint8_t y;
	for(y=0;y<8;y++)
	{
		CLKLOW;
		waitus(SERCLKDLY);
		byte |= GETIN << y;
		CLKHIZ;
		waitus(SERCLKDLY);
	}
	waitus(100);
	CLKLOW;	
	return byte;
}

/*Establish Sync*/
uint8_t Sync()
{
	uint16_t syncword=0;
//	uint16_t x;
//	syncword = 0;
//	for(x=0;x<512;x++)
	while(1)
	{
		CLKLOW;
		waitus(100);
//		syncword = syncword >> 1;
//		syncword |= GETIN << 15;	//these 2 lines take up huge amounts of space
		syncword <<=1;
		syncword |=GETIN;
		CLKHIZ;
		waitus(100);
//		if(syncword == 0xEEEE)
		if(syncword == 0x7777)
			break;
	}
	CLKLOW;
//	waitus(1000);
//	if(x < 512)
		return 0;
//	return -1;
}

/*Reaquire sync*/
void Resync()
{
	while(1)
	{
		if(Sync() == 0)
			break;
		waitus(3000);
	}
}

/*Activate a serial command*/
uint8_t SerialRequest(uint8_t command, uint32_t offset, uint8_t size, uint16_t data, uint16_t *ret)
{
	uint8_t x;
	uint8_t rb0,rb1;

	Buffer[0] = command;
	Buffer[1] = 0;
	Buffer[2] = (char) ((offset >> 8) & 0xFF);
	Buffer[3] = (char) (offset & 0xFF);
	Buffer[4] = (char) (data & 0xFF);
	Buffer[5] = (char) ((offset >> 16) & 0xFF);
	Buffer[6] = (char) ((data >> 8) & 0xFF);
	Buffer[7] = 0;
	Buffer[8] = size;
	Buffer[9] = 0;

	for(x=0;x<SERIAL_CMD_SIZE;x++)
	{
		TxByte(Buffer[x]);
	}
	waitus(1000);
	// For byte reads, zero the last result byte so the mask below won't contain old data
	rb1=0;
	// Get and check manditory part of the reply.
	if(RxByte() | RxByte()) 
		return -1;
	rb0=RxByte();
	if(size==2)
		rb1=RxByte();
	if(Buffer[0] == CMD_WRITE) //if write
	{ 
		if(Buffer[4] != rb0) //check that the first byte was written
			return -1;
		if( (size == 2) && (Buffer[6] != rb1))	// check that the next byte was written if writing 2 bytes (what about >2 bytes??)
			return -1;
	}	
	if(Buffer[0] == CMD_READ) // if reading copy the result buffer to the return buffer
	{
		*ret = ((uint16_t)rb0) | ((uint16_t)(rb1 << 8) & 0xFF00);
	}
	return 0;
}

// these are from an experiment to reduce progmem usage... 
// it works so long as u are using alot of writes.. but in current implementation is actually more wastefull
//void serialWriteByte(uint32_t offset, uint8_t data) {
//	SerialRequest(CMD_WRITE, offset, 1, data, NULL);
//}

//void serialWriteWord(uint32_t offset, uint16_t data) {
//	SerialRequest(CMD_WRITE, offset, 2, data, NULL);	
//}

/*Send out a block of data as efficiently as possible*/
void sendBlock(uint32_t offset, uint8_t *buff, uint16_t size) {
	SerialRequest(CMD_WRITE, UNLOCKADDR, 1, 0x41, NULL);		
//	serialWriteWord(UNLOCKADDR,0x4100);
	while(size>=2) { //if we find way to increase size, say to 4, change all these 2 to 4.
//		serialWriteWord(offset,pgm_read_word((PGM_P)buff));
		SerialRequest(CMD_WRITE, offset, 2, pgmReadWord((PGM_P)buff), NULL);
		offset+=2;
		buff+=2;
		size-=2;
	}
	if(size) 
//		serialWriteByte(offset,pgm_read_byte((PGM_P)buff));
		SerialRequest(CMD_WRITE, offset, size, pgmReadWord((PGM_P)buff), NULL); //this may need to be pgmReadByte ?
}
